#define INCL_WIN
#define INCL_GPI
#include <os2.h>
#include "bitview.h"


/* function prototypes */
INT main( VOID );
MRESULT EXPENTRY ClientWndProc( HWND, ULONG, MPARAM, MPARAM );
APIRET GetBMPFile( HWND, PHFILE );
APIRET ReadBitmap( HWND, HFILE );
VOID SaveFile( HWND );
VOID MessageBox ( HWND, PSZ );


/* global variables */
HAB hab;
HBITMAP hbm = 0;


INT main( VOID )
{
   HMQ   hmq;
   HWND  hwndFrame;
   HWND  hwndClient;
   CHAR  szClientClass[] = "Client Window";
   QMSG  qmsg;
   ULONG flCreateFlags = FCF_TITLEBAR      /* create: title bar control     */
                       | FCF_SYSMENU       /*         system menu control   */
                       | FCF_MENU          /*         action bar            */
                       | FCF_MINMAX        /*         min and max buttons   */
                       | FCF_SIZEBORDER    /* draw sizeborders              */
                       | FCF_SHELLPOSITION /* shell decides window position */
                       | FCF_TASKLIST      /* title added to switch list    */
                       | FCF_ICON;         /* frame has an icon             */

   hab = WinInitialize(0);
   hmq = WinCreateMsgQueue( hab, 0 );

   WinRegisterClass( hab,            /* anchor block handle */
                     szClientClass,  /* class name */
                     ClientWndProc,  /* pointer to window procedure */
                     CS_SIZEREDRAW,  /* class style */
                     0 );            /* number of class data bytes */

   hwndFrame = WinCreateStdWindow(
                HWND_DESKTOP,            /* Desktop window is parent     */
                WS_VISIBLE,              /* window styles                */
                &flCreateFlags,          /* frame control flag           */
                szClientClass,           /* client window class name     */
                "Bitmap Viewer",         /* title bar text         */
                0,                       /* no special class style       */
                NULLHANDLE,              /* resource is in .EXE file     */
                ID_FRAME_RES,            /* frame window identifier      */
                &hwndClient              /* client window handle         */
                );

   if ( hwndFrame )
      while( WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ) )
         WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow(hwndFrame);
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* end of main */


MRESULT EXPENTRY ClientWndProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   static SHORT sClientX, sClientY;  /* client window X and Y dimensions */

   switch( msg ) {

   case WM_COMMAND:

      switch (SHORT1FROMMP(mp1)) {

      case IDM_OPEN:
       { HFILE  FileHandle;
         APIRET rc;

         rc = GetBMPFile( hwnd, &FileHandle );
         if (rc == NO_ERROR) {
             rc = ReadBitMap(hwnd, FileHandle);
             if (rc == NO_ERROR)
                WinInvalidateRect(hwnd, NULL, FALSE);
             else
                MessageBox( hwnd, "Error reading bitmap." );
          }
        } break;

      case IDM_SAVE:
         SaveFile( hwnd );
         break;

      default:
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
      break;

   case WM_ERASEBACKGROUND:
      return (MRESULT)TRUE;

   case WM_PAINT:
    { HPS    hps;
      RECTL  rcl;

      hps = WinBeginPaint(hwnd, NULLHANDLE, &rcl);
      rcl.yBottom = rcl.xLeft = 0;
      rcl.xRight = (LONG)sClientX;
      rcl.yTop = (LONG)sClientY;
      WinDrawBitmap(hps,
                    hbm,
                    NULL,
                    (PPOINTL)&rcl,
                    0L,                   /* Fgnd color */
                    0L,                   /* Bkgd color */
                    DBM_STRETCH);
      WinEndPaint(hps);
    } break;

   case WM_SIZE:
      sClientX = SHORT1FROMMP(mp2);
      sClientY = SHORT2FROMMP(mp2);
      break;

   case WM_HELP:
      MessageBox(hwnd, "Help has been requested!");
      sClientY = SHORT2FROMMP(mp2);
      break;

   default:
      return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }

   return (MRESULT)FALSE;
} /* end of ClientWndProc */


APIRET GetBMPFile( HWND hwndClient, PHFILE pFileHandle )
{
   FILEDLG FileDlg;
   HWND hwndDlg;
   HFILE FileHandle;
   ULONG ulAction;
   APIRET rc = 1;

   memset( &FileDlg, 0, sizeof(FILEDLG) );  /* Init all fields to zero    */
   FileDlg.cbSize = sizeof(FILEDLG);
   FileDlg.fl = FDS_CENTER | FDS_HELPBUTTON | FDS_OPEN_DIALOG;
   FileDlg.pszTitle = "The Open File Dialog";
   strcpy(FileDlg.szFullFile, "*.bmp");

   hwndDlg = WinFileDlg(HWND_DESKTOP, hwndClient, (PFILEDLG)&FileDlg);

   if (hwndDlg && (FileDlg.lReturn == DID_OK)) {

      rc = DosOpen(
         FileDlg.szFullFile,                /* File name       */
         &FileHandle,                       /* Returned to us  */
         &ulAction,                         /* Action taken    */
         0,                                 /* File size (N/A) */
         FILE_NORMAL,                       /* File attributes */
         FILE_OPEN,
         OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE,
         NULL);                             /* EA buffer (N/A) */

      if(rc == NO_ERROR)
         *pFileHandle = FileHandle;
   }

   return rc;
} /* end of GetBMPFile */


APIRET ReadBitmap(HWND hwndClient, HFILE FileHandle )
{
   FILESTATUS FileInfo;
   PBYTE  pFileBegin = NULL;          /* beginning of bitmap file data    */
   ULONG  cbRead;                     /* Number of bytes read by DosRead. */
   HPS hps;
   PBITMAPFILEHEADER2 pbfh2;          /* can address any file types       */
   PBITMAPINFOHEADER2 pbmp2;
   ERRORID err;
   CHAR szBuffer[50];
   APIRET     rc;                     /* API return code                  */

   rc = DosQueryFileInfo(FileHandle,
                         FIL_STANDARD,        /* File information level   */
                         &FileInfo,
                         sizeof(FileInfo));
   if (rc)
      return(rc);

   rc = DosAllocMem((PPVOID) &pFileBegin,
                    (ULONG)  FileInfo.cbFile,
                    (ULONG)  PAG_READ | PAG_WRITE | PAG_COMMIT);

   if (rc)
      return(rc);

   rc = DosRead( FileHandle,
                 (PVOID)pFileBegin,
                 FileInfo.cbFile,
                 &cbRead);

   if (rc)
      return(rc);

   if( *((PUSHORT)pFileBegin) == BFT_BITMAPARRAY)
      pbfh2 = (PBITMAPFILEHEADER2)
                 &((PBITMAPARRAYFILEHEADER2)pFileBegin)->bfh2;

   else
      pbfh2 = (PBITMAPFILEHEADER2)pFileBegin;

   pbmp2 = &pbfh2->bmp2;    /* pointer to info header (readability) */

   hps = WinGetPS(hwndClient);

   hbm = GpiCreateBitmap(
            hps,
            pbmp2,                                /* BITMAPINFOHEADER2    */
            CBM_INIT,                             /* options              */
            (PBYTE)pFileBegin + pbfh2->offBits,   /* addr of bitmap data  */
            (PBITMAPINFO2)pbmp2);                 /* clr and format info  */

   if (!hbm)
   {
      err = WinGetLastError(hab);
      sprintf(szBuffer, "GpiLoadBitmap failed with a rc of %ld", err);
      MessageBox(hwndClient, szBuffer);
   }

   WinReleasePS(hps);

   DosFreeMem( pFileBegin);
   DosClose( FileHandle);

   return NO_ERROR;
} /* end of ReadBitmap */


VOID SaveFile( HWND hwndClient )
{
   FILEDLG FileDlg;
   HWND hwndDlg;

   memset( &FileDlg, 0, sizeof(FILEDLG) );  /* Init all fields to zero    */
   FileDlg.cbSize = sizeof(FILEDLG);
   FileDlg.fl = FDS_HELPBUTTON | FDS_SAVEAS_DIALOG | FDS_CENTER;
   FileDlg.pszTitle = "The Save File Dialog";
   strcpy(FileDlg.szFullFile, "*.bmp");

   hwndDlg = WinFileDlg(HWND_DESKTOP, hwndClient, (PFILEDLG)&FileDlg);

   if (hwndDlg && (FileDlg.lReturn == DID_OK))
      MessageBox( hwndClient, "Code to save BMP file not implemented." );

} /* end of SaveFile */


VOID MessageBox( HWND hwndOwner, PSZ pszText )
{

   WinMessageBox ( HWND_DESKTOP,
                   hwndOwner,
                   pszText,
                   "Bitview Example",        /* Message box title */
                   0,                        /* Message box ID    */
                   MB_OK
                 );
} /* end of MessageBox */
