#define INCL_WIN
#define INCL_GPI
#include <os2.h>
#include "bmpview.h"


/* prototypes */
INT main( VOID );
MRESULT EXPENTRY ClientWndProc( HWND, ULONG, MPARAM, MPARAM );
BOOL GetBMPFile( HWND );
VOID DisplayMessage( HWND, PSZ );
VOID ShowBMPInfo( HWND, LONG, SHORT );
VOID ShowBMPInformation( PBYTE, HWND );
VOID ShowBMPArrayFileHeader( PBITMAPARRAYFILEHEADER, HWND, HPS);
VOID ShowBMPFileHeader( PBITMAPFILEHEADER, HWND, HPS );
VOID ShowBMPInfoHeader( PBITMAPINFOHEADER, HWND, HPS );
VOID ShowBMPInfoHeader2( PBITMAPINFOHEADER2, HWND, HPS );


/* globals */
CHAR    szFileName[CCHMAXPATH];
CHAR    szBuffer[50];
SHORT   sClientY;
LONG    lLineHeight;
POINTL  pt;


INT main (VOID)
{

   HAB   hab;
   HWND  hwndFrame;
   HMQ   hmq;
   HWND  hwndClient;
   CHAR  szClientClass[] = "Client Window";
   QMSG  qmsg;
   ULONG flCreateFlags = FCF_TITLEBAR      /* create: title bar control     */
                       | FCF_SYSMENU       /*         system menu control   */
                       | FCF_MENU          /*         action bar            */
                       | FCF_MINMAX        /*         min and max buttons   */
                       | FCF_SIZEBORDER    /* draw sizeborders              */
                       | FCF_SHELLPOSITION /* shell decides window position */
                       | FCF_TASKLIST      /* title added to switch list    */
                       | FCF_ICON;         /* frame has an icon             */

   hab = WinInitialize(0);

   hmq = WinCreateMsgQueue( hab, 0 );

   WinRegisterClass( hab,            /* anchor block handle */
                     szClientClass,  /* class name */
                     ClientWndProc,  /* pointer to window procedure */
                     CS_SIZEREDRAW,  /* class style */
                     0 );            /* number of class data bytes */

   hwndFrame = WinCreateStdWindow(
                HWND_DESKTOP,            /* Desktop window is parent     */
                WS_VISIBLE,              /* window styles                */
                &flCreateFlags,          /* frame control flag           */
                szClientClass,           /* client window class name     */
                "BMP File Descriptions", /* title bar text               */
                0,                       /* no special class style       */
                NULLHANDLE,              /* resource is in .EXE file     */
                ID_FRAME_RES,            /* frame window identifier      */
                &hwndClient              /* client window handle         */
                );

   if ( hwndFrame )
      while (WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ))
         WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow( hwndFrame );   /* clean up */
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* end of main */


MRESULT EXPENTRY ClientWndProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   static BOOL fReady = FALSE;

   switch (msg) {

   case WM_CREATE:
    { HPS  hps;
      FONTMETRICS fm;

      hps = WinGetPS(hwnd);
      GpiQueryFontMetrics(hps, (LONG) sizeof fm, &fm);
      lLineHeight = fm.lMaxBaselineExt;
      WinReleasePS(hps);
    } break;

   case WM_COMMAND:

      switch (SHORT1FROMMP(mp1)) {

         case IDM_OPEN:
            if ( GetBMPFile(hwnd) ) {
               fReady = TRUE;
               ShowBMPInfo(hwnd, lLineHeight, sClientY);
            }
            break;

         default:
            return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
      break;

   case WM_ERASEBACKGROUND:
      /* return TRUE to request PM to paint the window background */
      return (MRESULT)TRUE;

   case WM_PAINT:
    { HPS    hps;
      RECTL  rctl;

      if (fReady) {
         hps = WinBeginPaint(hwnd, NULLHANDLE, &rctl);
         ShowBMPInfo(hwnd, lLineHeight, sClientY);
         WinEndPaint(hps);
      }
    } break;

   case WM_SIZE:
      sClientY = SHORT2FROMMP(mp2);
      break;

   default:
      return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }

   return (MRESULT)FALSE;
}  /* end of ClientWndProc */


APIRET GetBMPFile( HWND hwndClient )
{
   FILEDLG FileDlg;
   HWND hwndDlg;
   BOOL rc = FALSE;

   memset( &FileDlg, 0, sizeof(FILEDLG) );  /* Init all fields to zero */

   FileDlg.cbSize = sizeof(FILEDLG);

   FileDlg.fl = FDS_HELPBUTTON | FDS_CENTER | FDS_OPEN_DIALOG;

   FileDlg.pszTitle = "Open a BMP file";

   strcpy(FileDlg.szFullFile, "*.bmp");

   /* display the file dialog */
   hwndDlg = WinFileDlg(HWND_DESKTOP, hwndClient, (PFILEDLG)&FileDlg);
   WinAlarm(HWND_DESKTOP, WA_WARNING);

   /* upon sucessful return of a file, open it for reading */
   if (hwndDlg && (FileDlg.lReturn == DID_OK)) {

      strcpy(szFileName, FileDlg.szFullFile);
      return TRUE;
   } else
      return FALSE;

} /* end of GetBMPFile */


VOID ShowBMPInfo( HWND hwndClient, LONG lCharBoxY, SHORT sTotalY )
{

   HFILE       FileHandle;
   ULONG       ulAction;
   FILESTATUS  FileInfo;
   PBYTE       pFileBegin = NULL;  /* beginning of bitmap file data */
   ULONG       cbRead;             /* number of bytes read by DosRead. */
   APIRET      rc;                 /* API return code */

   rc = DosOpen( szFileName,             /* file name                      */
                 &FileHandle,            /* get file handle                */
                 &ulAction,              /* get status of file             */
                 0,                      /* ignored, used to set file size */
                 FILE_NORMAL,            /* file attribute bits            */
                 FILE_OPEN,              /* open flags                     */
                 OPEN_ACCESS_READONLY
                 | OPEN_SHARE_DENYNONE,  /* open mode                      */
                 NULL);                  /* no EA buffer                   */

   if (rc) {
      sprintf(szBuffer, "DosOpen failed with a rc of %ld", rc);
      DisplayMessage(hwndClient, szBuffer);
      return;
   }

   rc = DosQueryFileInfo( FileHandle,
                          FIL_STANDARD,  /* file information level */
                          &FileInfo,
                          sizeof(FileInfo));

   if (rc) {
      sprintf(szBuffer, "DosQueryFileInfo failed with a rc of %ld", rc);
      DisplayMessage(hwndClient, szBuffer);
      return;
   }

   rc = DosAllocMem((PPVOID) &pFileBegin,
                    (ULONG)  FileInfo.cbFile,
                    (ULONG)  PAG_READ | PAG_WRITE | PAG_COMMIT);

   if (rc) {
      sprintf(szBuffer, "DosAllocMem failed with a rc of %ld", rc);
      DisplayMessage(hwndClient, szBuffer);
      return;
   }

   rc = DosRead( FileHandle,
                 (PVOID)pFileBegin,
                 FileInfo.cbFile,
                 &cbRead );

   if (rc) {
      sprintf(szBuffer, "DosRead failed with a rc of %ld", rc);
      DisplayMessage(hwndClient, szBuffer);
      return;
   }

   ShowBMPInformation( pFileBegin, hwndClient);

   DosFreeMem( pFileBegin );
   DosClose( FileHandle );

} /* end of ShowBMPInfo */


VOID ShowBMPInformation( PBYTE pFileBegin, HWND hwndClient)
{

   HPS hps;
   PBITMAPARRAYFILEHEADER pBitmapArrayFileHeader;
   PBITMAPFILEHEADER pBitmapFileHeader;
   PBITMAPINFOHEADER2 pBitmapInfoHeader2;

   hps = WinGetPS(hwndClient);
   GpiErase(hps);

   pt.x = 5;
   pt.y = (LONG)sClientY - lLineHeight;

   if ( *((PUSHORT)pFileBegin) == BFT_BITMAPARRAY) {

      pBitmapArrayFileHeader = (PBITMAPARRAYFILEHEADER)pFileBegin;

      ShowBMPArrayFileHeader(pBitmapArrayFileHeader, hwndClient, hps);

      pFileBegin = (PBYTE)&pBitmapArrayFileHeader->bfh;
   }

   pBitmapFileHeader = (PBITMAPFILEHEADER)pFileBegin;

   if ( pBitmapFileHeader->usType == BFT_BMAP) {

      ShowBMPFileHeader(pBitmapFileHeader, hwndClient, hps);

      pBitmapInfoHeader2 = (PBITMAPINFOHEADER2)&pBitmapFileHeader->bmp;

      if ( pBitmapInfoHeader2->cbFix == sizeof(BITMAPINFOHEADER))
         ShowBMPInfoHeader( (PBITMAPINFOHEADER)pBitmapInfoHeader2,
                            hwndClient,
                            hps);

      else
         ShowBMPInfoHeader2(pBitmapInfoHeader2, hwndClient, hps);

   } else {
      sprintf(szBuffer, "Bitmap type %x not recognized!",(PUSHORT)pFileBegin);
      DisplayMessage(hwndClient, szBuffer);
   }

   WinReleasePS(hps);
} /* end of ShowBMPInformation */


VOID ShowBMPArrayFileHeader( PBITMAPARRAYFILEHEADER pArrayHeader,
                             HWND hwndClient,
                             HPS hps )
{

   sprintf( szBuffer, "BITMAPARRAYFILEHEADER.usType = %x",
            pArrayHeader->usType );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPARRAYFILEHEADER.cbSize = %ld",
            pArrayHeader->cbSize );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPARRAYFILEHEADER.offNext = %ld",
            pArrayHeader->offNext );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPARRAYFILEHEADER.cxDisplay = %d",
            pArrayHeader->cxDisplay );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPARRAYFILEHEADER.cyDisplay = %d",
            pArrayHeader->cyDisplay );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight*2;

} /* end of ShowBMPArrayFileHeader */


VOID ShowBMPFileHeader( PBITMAPFILEHEADER pBMapFileHdr,
                        HWND hwndClient,
                        HPS hps )
{

   sprintf( szBuffer, "BITMAPFILEHEADER.usType = %x", pBMapFileHdr->usType );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPFILEHEADER.cbSize = %ld", pBMapFileHdr->cbSize );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPFILEHEADER.xHotspot = %d",
            pBMapFileHdr->xHotspot );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPFILEHEADER.yHotspot = %d",
            pBMapFileHdr->yHotspot );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPFILEHEADER.offBits = %ld",
            pBMapFileHdr->offBits);
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight*2;

} /* end of ShowBMPFileHeader */


VOID ShowBMPInfoHeader( PBITMAPINFOHEADER pBMapInfoHdr,
                        HWND hwndClient,
                        HPS hps )
{

   sprintf(szBuffer, "BITMAPINFOHEADER.cbFix = %ld", pBMapInfoHdr->cbFix);
   GpiCharStringAt(hps, &pt, strlen(szBuffer), szBuffer);

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER.cx = %d", pBMapInfoHdr->cx );
   GpiCharStringAt(hps, &pt, strlen(szBuffer), szBuffer);

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER.cy = %d", pBMapInfoHdr->cy );
   GpiCharStringAt(hps, &pt, strlen(szBuffer), szBuffer);

   pt.y -= lLineHeight;
   sprintf(szBuffer, "BITMAPINFOHEADER.cPlanes = %d", pBMapInfoHdr->cPlanes);
   GpiCharStringAt(hps, &pt, strlen(szBuffer), szBuffer);

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER.cBitCount = %d",
            pBMapInfoHdr->cBitCount );
   GpiCharStringAt(hps, &pt, strlen(szBuffer), szBuffer);

} /* end of ShowBMPInfoHeader */


VOID ShowBMPInfoHeader2( PBITMAPINFOHEADER2 pBMapInfoHdr,
                         HWND hwndClient,
                         HPS hps)
{
   sprintf(szBuffer, "BITMAPINFOHEADER2.cbFix = %ld", pBMapInfoHdr->cbFix);
   GpiCharStringAt(hps, &pt, strlen(szBuffer), szBuffer);

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER2.cx = %d", pBMapInfoHdr->cx );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER2.cy = %d", pBMapInfoHdr->cy );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER2.cPlanes = %d",
            pBMapInfoHdr->cPlanes );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

   pt.y -= lLineHeight;
   sprintf( szBuffer, "BITMAPINFOHEADER2.cBitCount = %d",
            pBMapInfoHdr->cBitCount );
   GpiCharStringAt( hps, &pt, strlen(szBuffer), szBuffer );

} /* end of ShowBMPInfoHeader2 */


VOID DisplayMessage( HWND hwndOwner, PSZ pszText )
{

   WinMessageBox ( HWND_DESKTOP,
                   hwndOwner,
                   pszText,
                   "BMP Example",
                   0,
                   MB_OK );
} /* end of DisplayMessage */
