#define INCL_WIN
#define INCL_GPI
#include <os2.h>
#include "font.h"


/* function prototypes */
INT main( VOID );
MRESULT EXPENTRY ClientWndProc( HWND, ULONG, MPARAM, MPARAM );
VOID SetFont( HWND, PFONTDLG, HPS, HDC );
VOID DefaultOutput( SHORT, SHORT, HPS, HDC );


/* global variables */
HAB   hab;
CHAR  achFontname[FACESIZE];  /* buffer needed for Font dlg  */
LONG  lyFontResolution;


INT main( VOID )
{
   HMQ   hmq;
   HWND  hwndFrame;
   HWND  hwndClient;
   CHAR  szClientClass[] = "Client Window";
   QMSG  qmsg;
   ULONG flCreateFlags = FCF_TITLEBAR      /* create: title bar control     */
                       | FCF_SYSMENU       /*         system menu control   */
                       | FCF_MENU          /*         action bar            */
                       | FCF_MINMAX        /*         min and max buttons   */
                       | FCF_SIZEBORDER    /* draw sizeborders              */
                       | FCF_SHELLPOSITION /* shell decides window position */
                       | FCF_TASKLIST      /* title added to switch list    */
                       | FCF_ICON;         /* frame has an icon             */

   hab = WinInitialize(0);
   hmq = WinCreateMsgQueue( hab, 0 );

   WinRegisterClass( hab,            /* anchor block handle */
                     szClientClass,  /* class name */
                     ClientWndProc,  /* pointer to window procedure */
                     CS_SIZEREDRAW,  /* class style */
                     0 );            /* number of class data bytes */

   hwndFrame = WinCreateStdWindow(
                HWND_DESKTOP,            /* Desktop window is parent     */
                WS_VISIBLE,              /* window styles                */
                &flCreateFlags,          /* frame control flag           */
                szClientClass,           /* client window class name     */
                "Font Dialog Example",   /* title bar text               */
                0,                       /* no special class style       */
                NULLHANDLE,              /* resource is in .EXE file     */
                ID_FRAME_RES,            /* frame window identifier      */
                &hwndClient              /* client window handle         */
                );

   if ( hwndFrame )
      while( WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ) )
         WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow(hwndFrame);
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* end of main */


MRESULT EXPENTRY ClientWndProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   static HPS hps;                   /* Presentation Space handle       */
   static HDC hdc;                   /* Device Context handle           */
   static SHORT sClientX, sClientY;  /* Client wnd X and Y dimensions   */
   static USHORT  fPaint = 0;        /* Paint default string            */
   static FONTDLG fontdlg;

   switch( msg ) {

   case WM_CREATE:
    { SIZEL  sizel = { 0L, 0L };  /* presentation page size */
      hdc = WinOpenWindowDC(hwnd);

      hps = GpiCreatePS ( hab,
                          hdc,
                          &sizel,
                          PU_PELS
                          | GPIT_MICRO
                          | GPIA_ASSOC );

      memset(&fontdlg, 0, sizeof(FONTDLG));
      fontdlg.cbSize = sizeof(FONTDLG);

      fontdlg.hpsScreen = hps;

      achFontname[0] = '\0';
      fontdlg.pszFamilyname = achFontname;
      fontdlg.usFamilyBufLen = FACESIZE;
      fontdlg.clrFore = SYSCLR_WINDOWTEXT;
      fontdlg.clrBack = SYSCLR_WINDOW;
      fontdlg.fl = FNTS_CENTER | FNTS_INITFROMFATTRS;
      fontdlg.fAttrs.usRecordLength = sizeof(FATTRS);
    } break;

   case WM_COMMAND:

      switch (SHORT1FROMMP(mp1)) {

      case ID_FONT:
         fPaint = 0;
         SetFont(hwnd, &fontdlg, hps, hdc);
         WinInvalidateRect(hwnd, NULL, FALSE);
         break;

      default:
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
      break;

   case WM_ERASEBACKGROUND:
      return (MRESULT)TRUE;

   case WM_SIZE:
      sClientX = SHORT1FROMMP(mp2);
      sClientY = SHORT2FROMMP(mp2);
      break;

   case WM_PAINT:
    { RECTL  rcl;

      WinBeginPaint(hwnd, hps, &rcl);
      DefaultOutput(sClientX, sClientY, hps, hdc);
      WinEndPaint(hps);
    } break;

   case WM_DESTROY:
      GpiDestroyPS(hps);
      DevCloseDC(hdc);
      break;

   default:
      return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }

   return (MRESULT)FALSE;
} /* end of ClientWndProc */


VOID SetFont( HWND hwndOwner, PFONTDLG pfontdlg, HPS hps, HDC hdc )
{
   HWND      hwndFontDlg;
   CHAR      szFamilyname[FACESIZE];
   RECTL     rc;
   POINTL    pt;
   LONG      lMatch;
   BOOL      fSuccess;
   SIZEF     sizef;
   CHAR      achBuff[50];
   LONG      lxFontResolution;

   GpiErase(hps);

   hwndFontDlg = WinFontDlg(HWND_DESKTOP, hwndOwner, pfontdlg);

   if ((hwndFontDlg) && (pfontdlg->lReturn == DID_OK)) {

      GpiCreateLogFont( pfontdlg->hpsScreen,
                        NULL,
                        LCID_MYFONT,
                        &pfontdlg->fAttrs);

      GpiSetCharSet( pfontdlg->hpsScreen, LCID_MYFONT );

      if ( pfontdlg->fAttrs.fsFontUse == FATTR_FONTUSE_OUTLINE ) {
        DevQueryCaps( hdc, CAPS_HORIZONTAL_FONT_RES,
                      (LONG)1, &lxFontResolution);

        DevQueryCaps( hdc, CAPS_VERTICAL_FONT_RES,
                      (LONG)1, &lyFontResolution);

        /*  Calculate the size of the character box, based on the
         *  point size selected and the resolution of the device
         *  Pels = Inches * pels/inch
         */
        sizef.cx = (FIXED)(((pfontdlg->fxPointSize) / 72 ) *
                          lxFontResolution );
        sizef.cy = (FIXED)(((pfontdlg->fxPointSize) / 72 ) *
                          lyFontResolution );

        GpiSetCharBox( pfontdlg->hpsScreen, &sizef );

      }
   }
} /* end of SetFont */


VOID DefaultOutput( SHORT sTotalX, SHORT sTotalY, HPS hps, HDC hdc )
{
   FONTMETRICS fm;
   CHAR szStyle[20];
   LONG lPtSize;
   CHAR szString[80];
   POINTL aptl[TXTBOX_COUNT] ;
   POINTL pt;

   GpiErase(hps);

   GpiQueryFontMetrics(hps, sizeof(FONTMETRICS), &fm);

   if (fm.fsDefn & FM_DEFN_OUTLINE)
      strcpy(szStyle, "Outline");
   else
      strcpy(szStyle,"Image");

   DevQueryCaps( hdc, CAPS_VERTICAL_FONT_RES, (LONG)1, &lyFontResolution);

   lPtSize = ((fm.lEmHeight*72)+71) / lyFontResolution;
   sprintf(szString, "%s, %s, %ld point",
                      fm.szFamilyname,
                          szStyle,
                              lPtSize);

   GpiQueryTextBox (hps, strlen(szString), szString, TXTBOX_COUNT, aptl);

   if ((aptl[TXTBOX_BOTTOMRIGHT].x - aptl[TXTBOX_BOTTOMLEFT].x) < sTotalX)
      pt.x = (sTotalX/2) -
             ((aptl[TXTBOX_BOTTOMRIGHT].x - aptl[TXTBOX_BOTTOMLEFT].x)/2);
   else
      pt.x = 1;

   pt.y = sTotalY/2;
   GpiCharStringAt(hps, &pt, (LONG)strlen(szString), szString);

} /* end of DefaultOutput */
