#define INCL_WIN
#include <os2.h>
#include "icons.h"


/* function prototypes */
INT main( VOID );
MRESULT EXPENTRY MyFrameProc( HWND, ULONG, MPARAM, MPARAM );
VOID InitCnr( HWND );

/* global variables */
PFNWP        OldFrameWndProc;
HWND         hwndCnr;   /* container window handle */
HWND         hwndMenu;  /* context menu window handle */
SYSICONINFO  aIconInfo[NUM_SYS_ICONS] = {
   SPTR_ARROW,           "SPTR_ARROW",
   SPTR_TEXT,            "SPTR_TEXT",
   SPTR_WAIT,            "SPTR_WAIT",
   SPTR_SIZE,            "SPTR_SIZE",
   SPTR_MOVE,            "SPTR_MOVE",
   SPTR_SIZENWSE,        "SPTR_SIZENWSE",
   SPTR_SIZENESW,        "SPTR_SIZENESW",
   SPTR_SIZEWE,          "SPTR_SIZEWE",
   SPTR_SIZENS,          "SPTR_SIZENS",
   SPTR_APPICON,         "SPTR_APPICON",
   SPTR_ICONINFORMATION, "SPTR_ICONINFORMATION",
   SPTR_ICONQUESTION,    "SPTR_ICONQUESTION",
   SPTR_ICONERROR,       "SPTR_ICONERROR",
   SPTR_ICONWARNING,     "SPTR_ICONWARNING",
   SPTR_ILLEGAL,         "SPTR_ILLEGAL",
   SPTR_FILE,            "SPTR_FILE",
   SPTR_FOLDER,          "SPTR_FOLDER",
   SPTR_MULTFILE,        "SPTR_MULTFILE",
   SPTR_PROGRAM,         "SPTR_PROGRAM"
};


INT main (VOID)
{
   HAB  hab;                             /* PM anchor block handle        */
   HMQ  hmq;                             /* Message queue handle          */
   HWND hwndFrame;                       /* Frame window handle           */
   QMSG qmsg;                            /* Message from message queue    */
   FRAMECDATA fcd;

   fcd.cb = sizeof(FRAMECDATA);           /* length of structure */
   fcd.flCreateFlags =  FCF_TITLEBAR      /* create: title bar control    */
                      | FCF_SYSMENU       /*         system menu control  */
                      | FCF_SIZEBORDER    /*         sizeborders          */
                      | FCF_MINMAX        /*         min and max buttons  */
                      | FCF_SHELLPOSITION /* shell decides window pos     */
                      | FCF_TASKLIST;     /* title added to switch list   */
   fcd.hmodResources = NULLHANDLE;        /* handle to a module           */
   fcd.idResources   = 0;                 /* identifier for the frame     */

   hab = WinInitialize(0);

   hmq = WinCreateMsgQueue( hab, 0 );

   hwndFrame  = WinCreateWindow(
                   HWND_DESKTOP,    /* parent window is the desktop       */
                   WC_FRAME,        /* class name                         */
                   "System Icons",  /* text added to title bar            */
                   0,               /* window style                       */
                   0, 0, 0, 0,      /* ignored, FCF_SHELLPOSITION was set */
                   NULLHANDLE,      /* no owner window                    */
                   HWND_TOP,        /* top of Desktop's children          */
                   0,               /* ID number for frame resources      */
                   &fcd,            /* control data                       */
                   NULL );          /* no presentation parameters         */

  WinSendMsg(
     hwndFrame,
     WM_SETICON,
     (MPARAM)WinQuerySysPointer(HWND_DESKTOP, SPTR_ICONINFORMATION, FALSE),
     NULL);

   hwndCnr = WinCreateWindow(
                hwndFrame,         /* parent window handle       */
                WC_CONTAINER,      /* Window class               */
                NULL,              /* no window text             */
                CCS_AUTOPOSITION   /* window style               */
                | CCS_READONLY,
                0, 0, 0, 0,        /* frame formats client       */
                hwndFrame,         /* owner                      */
                HWND_BOTTOM,       /* Z-order                    */
                FID_CLIENT,        /* ID                         */
                NULL,              /* no control data            */
                NULL);             /* no presentation parms      */

   InitCnr(hwndCnr);

   OldFrameWndProc = WinSubclassWindow( hwndFrame, MyFrameProc);

   /* load context menu */
   hwndMenu = WinLoadMenu( HWND_OBJECT, NULLHANDLE, ID_CONTEXTMENU );

   WinShowWindow( hwndFrame, TRUE );

   if ( hwndFrame && hwndCnr )
      while( WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ) )
         WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow(hwndFrame);
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* end of main */


MRESULT EXPENTRY MyFrameProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{

   switch( msg ) {

     case WM_COMMAND:
      { CNRINFO cnrinfo;

        switch (SHORT1FROMMP(mp1)) {

           case IDM_ICONVIEW:
              cnrinfo.flWindowAttr = CV_ICON;
              break;

           case IDM_NAME_FLOWED:
              cnrinfo.flWindowAttr = CV_NAME | CV_FLOW;
              break;

           case IDM_NAME_NONFLOWED:
              cnrinfo.flWindowAttr = CV_NAME;
              break;

           case IDM_TEXT_FLOWED:
              cnrinfo.flWindowAttr = CV_TEXT | CV_FLOW;
              break;

           case IDM_TEXT_NONFLOWED:
              cnrinfo.flWindowAttr = CV_TEXT;
              break;

           default:
              return (MRESULT)(*OldFrameWndProc)( hwnd, msg, mp1, mp2 );
        }

        WinSendMsg( hwndCnr,
                    CM_SETCNRINFO,
                    &cnrinfo,
                    MPFROMLONG(CMA_FLWINDOWATTR)
                  );
      } break;

     case WM_CONTROL:

        switch (SHORT2FROMMP(mp1)) {

           case CN_CONTEXTMENU:
            { POINTL pt;

              WinQueryPointerPos(HWND_DESKTOP, &pt);

              WinPopupMenu( HWND_DESKTOP,
                            hwnd,
                            hwndMenu,
                            (SHORT)pt.x,
                            (SHORT)pt.y,
                            IDM_ICONVIEW,
                            PU_NONE | PU_SELECTITEM | PU_MOUSEBUTTON1 |
                            PU_MOUSEBUTTON2 | PU_KEYBOARD
                          );

            } return (MRESULT)FALSE;
        }
        break;
   }

   return (MRESULT)(*OldFrameWndProc)( hwnd, msg, mp1, mp2 );
} /* end of MyFrameProc */


VOID InitCnr(HWND hwnd)
{

   PRECORDCORE     pRecordCore, pStartingRecordCore;
   RECORDINSERT    recordInsert;
   PSYSICONINFO    pIconInfo;
   ULONG           i;

   pRecordCore = WinSendMsg(
                    hwnd,
                    CM_ALLOCRECORD,
                    0,                            /* No additional bytes */
                    MPFROMLONG(NUM_SYS_ICONS) );


   pStartingRecordCore = pRecordCore;
   pIconInfo = (PSYSICONINFO)&aIconInfo;

   for (i = 0; i < NUM_SYS_ICONS; i++) {

      pRecordCore->cb = sizeof(RECORDCORE);

      pRecordCore->hptrIcon = WinQuerySysPointer( HWND_DESKTOP,
                                                  pIconInfo->lIconID,
                                                  FALSE );

     pRecordCore->pszIcon = pRecordCore->pszName =
     pRecordCore->pszText = pIconInfo->pszIconName;

     pRecordCore = pRecordCore->preccNextRecord;
     ++pIconInfo;
   }

   recordInsert.cb = sizeof(RECORDINSERT);
   recordInsert.pRecordParent= NULL;
   recordInsert.pRecordOrder = (PRECORDCORE)CMA_END;
   recordInsert.zOrder = (USHORT)CMA_TOP;
   recordInsert.cRecordsInsert= NUM_SYS_ICONS;
   recordInsert.fInvalidateRecord = TRUE;

   WinSendMsg( hwnd,
               CM_INSERTRECORD,
               (PRECORDCORE)pStartingRecordCore,
               &recordInsert );

} /* end of InitCnr */
