#define INCL_WIN
#define INCL_WINSTDSLIDER
#define INCL_DOS
#define INCL_DOSPROCESS
#include <os2.h>
#include "mouseptr.h"


/* function prototypes */
MRESULT EXPENTRY MyDlgProc( HWND, ULONG, MPARAM, MPARAM );
VOID SetAcceleration( USHORT );


/* Acceleration Table
 *    The slider arm position is used as an index into this table.
 *    The table contains the level 1 and level 2 multipliers.
 */
                     /* M1 M2 */
ATABLE aTable[] = {      1, 1,
                         1, 1,
                         1, 2,
                         1, 3,
                         2, 3,
                         2, 4,
                         3, 5,
                         4, 6,
                         5, 7,
                         6, 8,
                         7, 10,
                  };


INT main( VOID )
{
   HAB   hab;
   HMQ   hmq;
   QMSG  qmsg;
   HWND  hwndDlg;
   SWP   swp;

   hab = WinInitialize( 0 );

   hmq = WinCreateMsgQueue( hab, 0 );

   hwndDlg = WinLoadDlg( HWND_DESKTOP,  /* set the desktop as parent      */
                         NULLHANDLE,    /* no owner                       */
                         MyDlgProc,     /* address of dialog procedure    */
                         NULLHANDLE,    /* use .EXE fill for resources    */
                         ID_MYDIALOG,   /* ID of the dialog box resource  */
                         NULL );        /* no control data                */

   /* find where system would place dialog box */
   WinQueryTaskSizePos( hab, 0, &swp );

   /* place dialog box at position given by the shell */
   WinSetWindowPos( hwndDlg,       /* change the dialog window's position */
                    NULLHANDLE,    /* ignored, SWP_ZORDER not set */
                    swp.x, swp.y,  /* position of lower left corner */
                    0, 0,          /* ignored SWP_SIZE not set */
                    SWP_SHOW       /* show dialog box */
                    | SWP_MOVE );  /* do not ignore swp.x and swp.y */

   if ( hwndDlg)
      while (WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ))
         WinDispatchMsg(hab, &qmsg);

   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* End of main */


MRESULT EXPENTRY MyDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   CHAR szBuffer[50];
   USHORT usArmPos;

   switch( msg ) {

      case WM_INITDLG:

         WinPostMsg( hwndDlg,
                     WM_SETICON,
                     (MPARAM)WinLoadPointer( HWND_DESKTOP,
                                             NULLHANDLE,
                                             IDI_MYICON ),
                     NULL);

         /* Set the slider arm to the center position (5) */
         WinSendDlgItemMsg ( hwndDlg,
                             ID_SLIDER,
                             SLM_SETSLIDERINFO,
                             MPFROM2SHORT( SMA_SLIDERARMPOSITION,
                                           SMA_INCREMENTVALUE ),
                             (MPARAM)CENTER_TICK );

         /* Set all of the tick marks 7 pixels high */
         WinSendDlgItemMsg ( hwndDlg,
                             ID_SLIDER,
                             SLM_SETTICKSIZE,
                             MPFROM2SHORT( SMA_SETALLTICKS, 7 ),
                             (MPARAM)NULL );

         /* Pad this ticks text or else it will get clipped */
         WinSendDlgItemMsg( hwndDlg,
                            ID_SLIDER,
                            SLM_SETSCALETEXT,
                            MPFROMSHORT( 0 ),
                            MPFROMP("Slow") );

         WinSendDlgItemMsg( hwndDlg,
                            ID_SLIDER,
                            SLM_SETSCALETEXT,
                            MPFROMSHORT( CENTER_TICK ),
                            MPFROMP("Default") );

         WinSendDlgItemMsg( hwndDlg,
                            ID_SLIDER,
                            SLM_SETSCALETEXT,
                            MPFROMSHORT( 10 ),
                            MPFROMP("Fast") );

         /* Get the number of pixels to the center of the shaft */
         usArmPos = (USHORT)WinSendDlgItemMsg(
                       hwndDlg,
                       ID_SLIDER,
                       SLM_QUERYSLIDERINFO,
                       MPFROM2SHORT( SMA_SLIDERARMPOSITION, SMA_RANGEVALUE ),
                       NULL );

         /* Place a detent in the exact center of the shaft */
         WinSendDlgItemMsg( hwndDlg,
                            ID_SLIDER,
                            SLM_ADDDETENT,
                            MPFROMSHORT( usArmPos ),
                            NULL );

         break;

      case WM_CONTROL:

         if ( SHORT1FROMMP(mp1) == ID_SLIDER ) {

            if (SHORT2FROMMP(mp1) == SLN_CHANGE)
               SetAcceleration( (USHORT)WinSendDlgItemMsg(
                                hwndDlg,
                                ID_SLIDER,
                                SLM_QUERYSLIDERINFO,
                                MPFROM2SHORT( SMA_SLIDERARMPOSITION,
                                              SMA_INCREMENTVALUE ),
                                (MPARAM)NULL ));
            break;
         } else
            return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

      case WM_COMMAND:
         switch( LOUSHORT(mp1) ) {

            case ID_CANCEL:
               SetAcceleration( CENTER_TICK );
               WinPostMsg( hwndDlg, WM_QUIT, 0, 0 );
               return FALSE;

            case ID_EXIT:
               WinPostMsg( hwndDlg, WM_QUIT, 0, 0 );
               return FALSE;

         } break;

      case WM_CLOSE:
            WinPostMsg( hwndDlg, WM_QUIT, 0, 0 );
            return FALSE;

      default:
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );

   }

   return (MRESULT)FALSE;
} /* end of MyDlgProc */


VOID SetAcceleration( USHORT usIndex )
{
   HFILE hMouse;
   ULONG ulAction, ulParmInOut = 0, ulDataInOut = 0;
   THRESHOLD th;

   DosOpen( (PSZ)"MOUSE$",
            (PHFILE)&hMouse,
            (PULONG)&ulAction,
            (ULONG)0L,
            (ULONG)FILE_READONLY,
            (ULONG)FILE_OPEN,
            (ULONG)OPEN_SHARE_DENYNONE,
            (PVOID)NULL
          );

   th.Length = 10;
   th.Level1  = 5;
   th.Level2  = 10;

   th.Mult1  = aTable[usIndex].m1;
   th.Mult2  = aTable[usIndex].m2;

   /*  This IOCTL will set the new threshold values */

   DosDevIOCtl( (HFILE)hMouse,
                (ULONG)0x0007,
                (ULONG)0x0055,
                (PVOID)&th,
                (ULONG)sizeof(THRESHOLD),
                (PULONG)&ulParmInOut,
                (PVOID)NULL,
                (ULONG)0L,
                (PULONG)&ulDataInOut
              );

   DosClose(hMouse) ;
} /* end of SetAcceleration */
