#define INCL_WIN
#define INCL_GPI
#define INCL_DEV
#define INCL_DOS
#include <os2.h>
#include "palette.h"


/* prototypes */
INT main( VOID );
MRESULT EXPENTRY ClientWndProc( HWND, ULONG, MPARAM, MPARAM );


/* globals */
HAB  hab;
RGB2 Palette[128];


INT main (VOID)
{
   HMQ   hmq;
   HWND  hwndFrame;
   HWND  hwndClient;
   QMSG  qmsg;
   CHAR  szClientClass[] = "Client Window";
   ULONG flCreateFlags = FCF_TITLEBAR      /* create: title bar control     */
                       | FCF_SYSMENU       /*         system menu control   */
                       | FCF_MENU          /*         action bar            */
                       | FCF_MINMAX        /*         min and max buttons   */
                       | FCF_SIZEBORDER    /* draw sizeborders              */
                       | FCF_SHELLPOSITION /* shell decides window position */
                       | FCF_TASKLIST      /* title added to switch list    */
                       | FCF_ICON;         /* frame has an icon             */

   hab = WinInitialize(0);

   hmq = WinCreateMsgQueue( hab, 0 );

   WinRegisterClass( hab,            /* anchor block handle */
                     szClientClass,  /* class name */
                     ClientWndProc,  /* pointer to window procedure */
                     CS_SIZEREDRAW,  /* class style */
                     0 );            /* number of class data bytes */

   hwndFrame = WinCreateStdWindow(
                  HWND_DESKTOP,       /* desktop window is parent     */
                  WS_VISIBLE,         /* window styles                */
                  &flCreateFlags,     /* frame control flag           */
                  szClientClass,      /* client window class name     */
                  "Palette Example",  /* no window text               */
                  0,                  /* no special class style       */
                  NULLHANDLE,         /* resource is in .EXE file     */
                  ID_FRAME_RES,       /* frame window identifier      */
                  &hwndClient         /* client window handle         */
                  );

   if ( hwndFrame )
      while (WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ))
         WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow( hwndFrame );   /* clean up */
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* end of main */


MRESULT EXPENTRY ClientWndProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   static HDC    hdc;                 /* device context handle     */
   static HPS    hps;                 /* presentation Space handle */
   static HPAL   hpal;                /* palette handle            */
   static SHORT  cxClient, cyClient;  /* size of the client window */

   /* variables to keep handy for storing temporary values */
   static ULONG  ulClrChanged;        /* count of colors changed   */
   static LONG   i;                   /* used for array indexing   */


   switch( msg ) {

   case WM_CREATE:
    { SIZEL  pagesize = { 0L, 0L }; /* presentation page size    */
      LONG   lReturn;

      /* Since we are using a cached micro PS we can simply use this
       *    call to obtain a handle to the display device context.
       */
      hdc = WinOpenWindowDC(hwnd);

      DevQueryCaps( hdc,
                    CAPS_ADDITIONAL_GRAPHICS,
                    (LONG)1,
                    &lReturn );

      if (!(lReturn & CAPS_PALETTE_MANAGER))
         WinMessageBox( HWND_DESKTOP,           /* parent window handle */
                        hwnd,                   /* owner window handle  */
                        "No Palette Support",   /* MsgBox Text          */
                        "Palette Program",      /* MsgBox Title Text    */
                        0,                      /* MsgBox Window ID     */
                        MB_ENTER | MB_WARNING); /* MsgBox window style  */

      hps = GpiCreatePS ( hab,
                          hdc,
                          &pagesize,
                          PU_PELS |
                          GPIT_MICRO |
                          GPIA_ASSOC );


      /* Set up the RGB values in our logical palette (alLogPal).
       * Our logical palette is an array of 65 LONGs which contain
       * the RGB values. Each color is a 4 byte integer, with a value
       * (F*16777216) + (R*65536) + (G*256) + B
       */
      for (i = 0; i < 128; i++) {
         Palette[i].bBlue = (BYTE)i;
         Palette[i].bGreen = Palette[i].bRed = Palette[i].fcOptions = 0;
      } /* endfor */

      hpal = GpiCreatePalette( hab,
                               LCOL_PURECOLOR,
                               LCOLF_CONSECRGB,  /* array of RGB values */
                               128,              /* count of elements   */
                               (PULONG)&Palette );

      GpiSelectPalette(hps, hpal);

      WinRealizePalette(hwnd, hps, &ulClrChanged);

    } break;

   case WM_COMMAND:

      switch (SHORT1FROMMP(mp1)) {

         case IDM_BLUE:
         case IDM_RED:
         case IDM_GREEN:
         case IDM_GRAY:

            if ( SHORT1FROMMP(mp1) == IDM_BLUE ) {
               for (i = 0; i < 128; i++) {
                  Palette[i].bBlue = (BYTE)i;
                  Palette[i].bGreen = Palette[i].bRed
                      = Palette[i].fcOptions = 0;
               } /* endfor */
            }

            else if ( SHORT1FROMMP(mp1) == IDM_RED ) {
               for (i = 0; i < 128; i++) {
                  Palette[i].bRed = (BYTE)i;
                  Palette[i].bGreen = Palette[i].bBlue
                     = Palette[i].fcOptions = 0;
               } /* endfor */
            }

            else if ( SHORT1FROMMP(mp1) == IDM_GREEN ) {
               for (i = 0; i < 128; i++) {
                  Palette[i].bGreen = (BYTE)i;
                  Palette[i].bRed = Palette[i].bBlue
                     = Palette[i].fcOptions = 0;
               } /* endfor */
            }

            else if ( SHORT1FROMMP(mp1) == IDM_GRAY ) {
               for (i = 0; i < 128; i++) {
                  Palette[i].bBlue = Palette[i].bGreen =
                  Palette[i].bRed = (BYTE)i;
                  Palette[i].fcOptions = 0;
               } /* endfor */
            }

            GpiSetPaletteEntries( hpal,
                                  LCOLF_CONSECRGB, /* array of RGB values */
                                  0,               /* starting index      */
                                  128,             /* count of elements   */
                                  (PULONG)&Palette );

            WinRealizePalette(hwnd, hps, &ulClrChanged);
            WinInvalidateRect(hwnd, NULL, FALSE);
            break;

         default:
            return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
      break;

   case WM_ERASEBACKGROUND:
      return (MRESULT)( TRUE );

   case WM_PAINT:
    { RECTL        rcl;

      WinBeginPaint( hwnd, hps, &rcl );

      for (i = 0; i < 128; i++) {
         rcl.yBottom = 0;
         rcl.yTop = cyClient;
         rcl.xLeft = i * (LONG)cxClient/128;
         rcl.xRight = (i + 1) * (LONG)cxClient/128;

         WinFillRect( hps, (PRECTL)&rcl, i );

      } /* endfor */

      WinEndPaint( hps );
    } break;

   case WM_SIZE:
      cxClient = SHORT1FROMMP(mp2);
      cyClient = SHORT2FROMMP(mp2);
      break;

   case WM_REALIZEPALETTE:
      if ( WinRealizePalette(hwnd, hps, &ulClrChanged) )
         WinInvalidateRect(hwnd, NULL, FALSE);
      break;

   case WM_DESTROY:
      GpiDestroyPS(hps);
      DevCloseDC(hdc);
      break;

   default:
      return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }

   return (MRESULT)FALSE;
} /* end of ClientWndProc */
