#define INCL_GPI
#define INCL_WIN
#include <os2.h>
#include <stdlib.h>
#include <string.h>
#include "winlist.h"


/* function prototypes */
INT main( VOID );
MRESULT EXPENTRY WindowProc(HWND, ULONG, MPARAM, MPARAM);
SHORT AddToMyListBox( HWND, HWND, SHORT, SHORT );


/* structures */
typedef struct _MYLISTITEM {
   HWND   hwnd;
   SHORT  level;
} MYLISTITEM;


/* global variables */
HAB  hab;
CLASSINFO  fci;
MYLISTITEM desktop = { HWND_DESKTOP, 0 };


INT main( VOID )
{
   HMQ  hmq;
   QMSG qmsg;
   CHAR szMyFrameClass[] = "Window List";
   HWND hwndFrame;
   FRAMECDATA fcd;

   fcd.cb = sizeof(FRAMECDATA);  /* length of structure */
   fcd.flCreateFlags =  FCF_TITLEBAR      /* create: title bar control     */
                      | FCF_SYSMENU       /*         system menu control   */
                      | FCF_MINMAX        /*         min and max buttons   */
                      | FCF_SIZEBORDER    /* draw sizeborders              */
                      | FCF_SHELLPOSITION /* shell decides window position */
                      | FCF_TASKLIST      /* title added to switch list    */
                      | FCF_ICON;         /* frame has an icon             */
   fcd.hmodResources = NULLHANDLE;  /* refer to this exe file */
   fcd.idResources   = ID_WINLIST;  /* identifier for the frame's resources */

   hab = WinInitialize((ULONG)NULL);
   hmq = WinCreateMsgQueue(hab, 0);

   WinQueryClassInfo( hab, WC_FRAME, &fci );

   fci.flClassStyle &= ~CS_PUBLIC;

   WinRegisterClass( hab,                 /* anchor block handle */
                     szMyFrameClass,      /* class name */
                     WindowProc,          /* pointer to window procedure */
                     fci.flClassStyle,    /* class style */
                     fci.cbWindowData );  /* number of class data bytes */

   hwndFrame  = WinCreateWindow(
                   HWND_DESKTOP,   /* parent window is the desktop */
                   szMyFrameClass, /* class name */
                   "PM Window Lister",  /* text added to title bar */
                   0,              /* window style */
                   0, 0, 0, 0,     /* ignored, FCF_SHELLPOSITION was set */
                   NULLHANDLE,     /* no owner window */
                   HWND_TOP,       /* place on top of Desktop's children */
                   ID_WINLIST,     /* ID number for frame resources */
                   &fcd,           /* data unique for frame windows */
                   NULL );         /* no presentation parameters */

   WinShowWindow( hwndFrame, TRUE );

   if ( hwndFrame )
      while (WinGetMsg( hab, &qmsg, NULLHANDLE, 0, 0 ))
         WinDispatchMsg( hab, &qmsg );

   WinDestroyWindow( hwndFrame );
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   return 0;
} /* end main */


MRESULT EXPENTRY WindowProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   static HWND hwndClient;

   switch (msg) {
   case WM_CREATE:
      /* call the frame class window procedure */
      (*fci.pfnWindowProc)( hwnd, msg, mp1, mp2 );

      /* create a list box as a client window */
      hwndClient = WinCreateWindow(
                      hwnd,          /* parent window is the frame */
                      WC_LISTBOX,    /* class name */
                      NULL,          /* window name not used by list boxes */
                      LS_NOADJUSTPOS /* window style */
                      | LS_HORZSCROLL,
                      0, 0, 0, 0,   /* ignored, frame positions this window */
                      hwnd,         /* the frame window is the owner */
                      HWND_BOTTOM,  /* standard z-order for a client window */
                      FID_CLIENT,   /* ID number recognized by the frame */
                      NULL,         /* no control data */
                      NULL );       /* no presentation parameters */

      /* Mention the Desktop in the list box */
      AddToMyListBox( hwndClient, HWND_DESKTOP, 0, 0 );

      break;

   case WM_CONTROL:
    { HWND  hwndParent;
      HWND  hwndNext;
      HENUM henum;
      SHORT Index;
      SHORT Level;
      MYLISTITEM * pItem;

      /* only interested in a LN_ENTER message from client window */
      if ((SHORT1FROMMP(mp1) == FID_CLIENT)&&(SHORT2FROMMP(mp1) == LN_ENTER))
      {
         Index = SHORT1FROMMR( WinSendMsg( hwndClient, LM_QUERYSELECTION,
                                           NULL, NULL ));
         pItem = (MYLISTITEM *)WinSendMsg( hwndClient, LM_QUERYITEMHANDLE,
                                           MPFROMSHORT(Index), NULL );
         Index++; /* insert after the item selected */
         Level = pItem->level + 1; /* indent the text */
         hwndParent = pItem->hwnd;

         /* remove and add items without redrawing */
         WinEnableWindowUpdate( hwndClient, FALSE );

         /* existing items of child windows may be out-dated, remove them */
         /*    also we do not want the list box to grow out of control    */
         while ( SHORT1FROMMR( WinSendMsg( hwndClient, LM_QUERYITEMCOUNT,
                                           NULL, NULL)) > Index )
            if ((pItem=(MYLISTITEM *)WinSendMsg( hwndClient,
                                                   LM_QUERYITEMHANDLE,
                                                   MPFROMSHORT(Index),
                                                   NULL ))->level >= Level)
            {
              WinSendMsg(hwndClient, LM_DELETEITEM, MPFROMSHORT(Index), NULL);
              free( pItem );
            } else
              break;/* the remaining windows are not children of hwndParent */

         /* add items about existing windows */
         henum = WinBeginEnumWindows( hwndParent );
         while ((hwndNext = WinGetNextWindow(henum)) != NULLHANDLE)
            AddToMyListBox( hwndClient, hwndNext, Index++, Level );
         WinEndEnumWindows (henum);

         /* show the updated list box */
         WinShowWindow( hwndClient, TRUE );
      } /* endif */
    } break;

   default:
      return (*fci.pfnWindowProc)( hwnd, msg, mp1, mp2 );
   } /* endswitch */

   return FALSE;
} /* end of WindowProc */


SHORT AddToMyListBox( HWND hwnd, HWND hwndTarget, SHORT Index, SHORT Level )
{
   MYLISTITEM * pItem;
   CHAR   szWindowText[99];
   CHAR   szClassName[99];
   PSZ    pszBuffer = NULL;
   PSZ    psz;
   SWP    swp;
   SHORT  i, rc;

   if (hwndTarget == HWND_DESKTOP) {
      pItem = &desktop;
      strcpy( szWindowText, "<Desktop Window>" );
   } else {
      pItem = malloc(sizeof(MYLISTITEM));
      if (!pItem)
         goto InvalidHandle;
      pItem->hwnd  = hwndTarget;
      pItem->level = Level;
      WinQueryWindowText(hwndTarget, sizeof(szWindowText), szWindowText );
   } /* endif */

   if (!WinQueryClassName(  hwndTarget, sizeof(szClassName), szClassName ))
      goto InvalidHandle;

   if (!WinQueryWindowPos(  hwndTarget, &swp ))
      goto InvalidHandle;

   pszBuffer = malloc( Level + 300 );
   if (!pszBuffer)
      goto InvalidHandle;

   for ( i=0, psz=pszBuffer; i<(3*Level); i++, psz++ )
      *psz = ' ';
   sprintf( psz, "\"%s\" class=\"%s\" pos=(%ld,%ld) width=%ld height=%ld",
                 szWindowText, szClassName, swp.x, swp.y, swp.cx, swp.cy );
   rc=(SHORT)WinSendMsg( hwnd, LM_INSERTITEM, MPFROMSHORT(Index), pszBuffer );
   WinSendMsg( hwnd, LM_SETITEMHANDLE, MPFROMSHORT(Index), pItem );
   free( pszBuffer );

   return rc;

InvalidHandle:
   free( pszBuffer );
   free( pItem );
   return FALSE;
} /* end of AddToMyListBox */

